<?php

if (!defined('ABSPATH')) exit;

class fenoticesAdmin {
    function __construct() {
        add_action('add_meta_boxes', [$this, 'register_custom_fields']);
        add_action('save_post', [$this, 'save_your_meta_box_data']);

        add_action( 'init', [$this, 'cptui_register_my_cpts'] );
    }

    function cptui_register_my_cpts() {

        /**
         * Post Type: Frontend Notice.
         */
    
        $labels = [
            "name" => esc_html__( "Frontend Notice", "frontend-notices" ),
            "singular_name" => esc_html__( "Frontend Notice", "frontend-notices" ),
            "menu_name" => esc_html__( "Notices", "frontend-notices" ),
            "all_items" => esc_html__( "All Notices", "frontend-notices" ),
            "add_new" => esc_html__( "Add new", "frontend-notices" ),
            "add_new_item" => esc_html__( "Add new notice", "frontend-notices" ),
            "edit_item" => esc_html__( "Edit notice", "frontend-notices" ),
            "new_item" => esc_html__( "New notice", "frontend-notices" ),
            "view_item" => esc_html__( "View notice", "frontend-notices" ),
            "view_items" => esc_html__( "View Frontend Notice", "frontend-notices" ),
            "search_items" => esc_html__( "Search Frontend Notice", "frontend-notices" ),
            "not_found" => esc_html__( "No Frontend Notice found", "frontend-notices" ),
            "not_found_in_trash" => esc_html__( "No Frontend Notice found in trash", "frontend-notices" ),
            "parent" => esc_html__( "Parent Frontend Notice:", "frontend-notices" ),
            "featured_image" => esc_html__( "Featured image for this Frontend Notice", "frontend-notices" ),
            "set_featured_image" => esc_html__( "Set featured image for this Frontend Notice", "frontend-notices" ),
            "remove_featured_image" => esc_html__( "Remove featured image for this Frontend Notice", "frontend-notices" ),
            "use_featured_image" => esc_html__( "Use as featured image for this Frontend Notice", "frontend-notices" ),
            "archives" => esc_html__( "Frontend Notice archives", "frontend-notices" ),
            "insert_into_item" => esc_html__( "Insert into Frontend Notice", "frontend-notices" ),
            "uploaded_to_this_item" => esc_html__( "Upload to this Frontend Notice", "frontend-notices" ),
            "filter_items_list" => esc_html__( "Filter Frontend Notice list", "frontend-notices" ),
            "items_list_navigation" => esc_html__( "Frontend Notice list navigation", "frontend-notices" ),
            "items_list" => esc_html__( "Frontend Notice list", "frontend-notices" ),
            "attributes" => esc_html__( "Frontend Notice attributes", "frontend-notices" ),
            "name_admin_bar" => esc_html__( "Frontend Notice", "frontend-notices" ),
            "item_published" => esc_html__( "Frontend Notice published", "frontend-notices" ),
            "item_published_privately" => esc_html__( "Frontend Notice published privately.", "frontend-notices" ),
            "item_reverted_to_draft" => esc_html__( "Frontend Notice reverted to draft.", "frontend-notices" ),
            "item_trashed" => esc_html__( "Frontend Notice trashed.", "frontend-notices" ),
            "item_scheduled" => esc_html__( "Frontend Notice scheduled", "frontend-notices" ),
            "item_updated" => esc_html__( "Frontend Notice updated.", "frontend-notices" ),
            "parent_item_colon" => esc_html__( "Parent Frontend Notice:", "frontend-notices" ),
        ];
    
        $args = [
            "label" => esc_html__( "Frontend Notice", "frontend-notices" ),
            "labels" => $labels,
            "description" => "",
            "public" => true,
            "publicly_queryable" => false,
            "show_ui" => true,
            "show_in_rest" => true,
            "rest_base" => "",
            "rest_controller_class" => "WP_REST_Posts_Controller",
            "rest_namespace" => "wp/v2",
            "has_archive" => false,
            "show_in_menu" => true,
            "show_in_nav_menus" => true,
            "delete_with_user" => false,
            "exclude_from_search" => true,
            "capability_type" => "post",
            "map_meta_cap" => true,
            "hierarchical" => false,
            "can_export" => false,
            "rewrite" => [ "slug" => "fen-frontendnotice", "with_front" => true ],
            "query_var" => true,
            "supports" => [ "title", "editor" ],
            "show_in_graphql" => false,
        ];
    
        register_post_type( "fen-frontendnotice", $args );
    }
    

    function register_custom_fields() {
        // Add a meta box to the 'your_custom_post_type' post type
        add_meta_box(
            'fenotices_settings', // Meta box ID
            __('Frontend Notices', 'frontend-notices'), // Meta box title
            [$this, 'custom_fields_callback'], // Callback function to display the box content
            'fen-frontendnotice', // Custom Post Type name
            'normal', // Context: 'normal', 'side', 'advanced'
            'high' // Priority: 'high', 'core', 'default', 'low'
        );
    }

    function custom_fields_callback($post) {
        // Use nonce for verification (security)
        wp_nonce_field('save_fenotices_settings_data', 'fenotices_settings_nonce');
    
        // Get the existing value from the database
        $location = get_post_meta($post->ID, 'fenotices_display_location', true);
        $is_dismissable = get_post_meta($post->ID, 'fenotices_notice_dismissable', true);
        $show_users = get_post_meta($post->ID, 'fenotices_show_users', true);
        $user_roles = get_post_meta($post->ID, 'fenotices_user_roles', true);
        $fenotices_post_types = get_post_meta($post->ID, 'fenotices_post_types', true);
        $mobile_only = get_post_meta($post->ID, 'fenotices_mobile_only', true);

        // Set defaults
        if (empty($location)) {
            $location = 'beforeheader';
        }

        if (empty($is_dismissable)) {
            $is_dismissable = false;
        }

        if (empty($show_users)) {
            $show_users = 'allusers';
        }

        if (empty($user_roles)) {
            $user_roles = ['allroles'];
        }

        if (empty($fenotices_post_types)) {
            $fenotices_post_types = ['all'];
        }
        
        if ($mobile_only === null) {
            $mobile_only = false;
        }
    
        // Display the field for entering data
        ?>
        <h4><?php esc_html_e('General', 'frontend-notices'); ?></h3>
        <p>
            <div style="width:50%;display:inline-block;">
                <label for="fenotices_display_location"><?php esc_html_e('Display Location:', 'frontend-notices'); ?></label>
                <p class="description">
                    <?php esc_html_e('Choose where the notice should be displayed.', 'frontend-notices'); ?>
                </p>
            </div>
            <div class="field" style="display:inline-block;vertical-align:top;">
                <select name="fenotices_display_location" id="fenotices_display_location"  value="<?php echo esc_attr($location); ?>">
                    <option <?php echo $location == '' ? 'selected' : ''; ?> disabled>Select display location</option>
                    <option <?php echo $location == 'beforeheader' ? 'selected' : ''; ?> value="beforeheader">Before header</option>
                    <option <?php echo $location == 'beforewoocommercecart' ? 'selected' : ''; ?> value="beforewoocommercecart">Before WooCommerce cart</option>
                    <option <?php echo $location == 'beforewoocommercecheckout' ? 'selected' : ''; ?> value="beforewoocommercecheckout">Before WooCommerce checkout</option>
                </select>
            </div>
        </p>
        <p>
            <div style="width:50%;display:inline-block;">
                <label for="fenotices_display_location"><?php esc_html_e('Select Post Types for Notices:', 'frontend-notices'); ?></label>
                <p class="description">
                    <?php esc_html_e('Choose which post types to display the notice on.', 'frontend-notices'); ?>
                </p>
            </div>
            <div class="field" style="display:inline-block;vertical-align:top;">
                <label for="">
                    <input type="checkbox" name="fenotices_post_types[]" id="fenotices_user_roles" <?php checked(1, in_array('all', $fenotices_post_types), true); ?> value="all">
                    All post types
                </label>
                <br>

                <?php
                $post_types = get_post_types(['public' => true, 'exclude_from_search' => false], 'objects');
                foreach ($post_types as $post_type) {
                    ?>
                    <label for="">
                        <input type="checkbox" name="fenotices_post_types[]" id="fenotices_user_roles" <?php checked(1, in_array($post_type->name, $fenotices_post_types), true); ?> value="<?php echo esc_attr($post_type->name); ?>">
                        <?php echo esc_html($post_type->name); ?>
                    </label>
                    <br>
                    <?php
                }
                ?>
            </div>
        </p>
        <p>
            <div style="width:50%;display:inline-block;">
                <label for="fenotices_notice_dismissable"><?php esc_html_e('Show Notice on Mobile Only', 'frontend-notices'); ?></label>
                <p class="description">
                    <?php esc_html_e('Enable this option to display notices exclusively on mobile devices, ensuring desktop users won\'t see them.', 'frontend-notices'); ?>
                </p>
            </div>
            <div class="field" style="display:inline-block;vertical-align:top;">
                <input type="checkbox" name="fenotices_mobile_only" id="fenotices_mobile_only" <?php checked(1, $mobile_only, true); ?> > <label for="">Enable</label>
            </div>
        </p>
        <h4><?php esc_html_e('User Targeting', 'frontend-notices'); ?></h4>
        <p>
            <div style="width:50%;display:inline-block;">
                <label for="fenotices_show_users"><?php esc_html_e('Show to Users:', 'frontend-notices'); ?></label>
                <p class="description">
                    <?php esc_html_e('Specify which users should see this notice.', 'frontend-notices'); ?>
                </p>
            </div>
            <div class="field" style="display:inline-block;vertical-align:top;">
                <select name="fenotices_show_users" id="fenotices_show_users">
                    <option <?php echo $show_users == 'allusers' ? 'selected' : ''; ?> value="allusers">All Users</option>
                    <option <?php echo $show_users == 'loggedin' ? 'selected' : ''; ?> value="loggedin">Logged-in Users</option>
                    <option <?php echo $show_users == 'loggedout' ? 'selected' : ''; ?> value="loggedout">Logged-out Users</option>
                </select>
            </div>
        </p>
        <p>
            <div style="width:50%;display:inline-block;">
                <label for="fenotices_user_roles"><?php esc_html_e('User Roles:', 'frontend-notices'); ?></label>
                <p class="description">
                    <?php esc_html_e('Display the notice based on user roles:', 'frontend-notices'); ?>
                </p>
            </div>
            <div class="field" style="display:inline-block;vertical-align:top;">

                <label for="">
                    <input type="checkbox" name="fenotices_user_roles[]" id="fenotices_user_roles" <?php checked(1, in_array('allroles', $user_roles), true); ?> value="allroles">
                    All roles
                </label>
                <br>
                
                <?php
                $roles = wp_roles()->roles;
                
                // Loop through each role
                foreach ($roles as $role_key => $role_details) {
                    ?>
                    <label for="">
                        <input type="checkbox" name="fenotices_user_roles[]" id="fenotices_user_roles" <?php checked(1, in_array($role_key, $user_roles), true); ?> value="<?php echo esc_attr($role_key); ?>">
                        <?php echo esc_html( $role_details['name'] ); ?>
                    </label>
                    <br>
                    <?php
                }
                ?>

            </div>
        </p>
        <?php
    }

    function save_your_meta_box_data($post_id) {
        // Check if nonce is set
        if (empty($_POST['fenotices_settings_nonce']) || sanitize_text_field( wp_unslash($_POST['fenotices_settings_nonce'])) === null) {
            return;
        }
    
        // Verify the nonce
        if (empty($_POST['fenotices_settings_nonce']) || !wp_verify_nonce(sanitize_text_field( wp_unslash($_POST['fenotices_settings_nonce']) ), 'save_fenotices_settings_data')) {
            return;
        }
    
        // Check for auto-save and user capabilities
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }
        if (!current_user_can('edit_post', $post_id)) {
            return;
        }
    
        if (!empty($_POST['fenotices_display_location'])) {
            $location = sanitize_text_field( wp_unslash( $_POST['fenotices_display_location'] ) );
            $possbile_values = ['beforeheader', 'beforewoocommercecart', 'beforewoocommercecheckout'];

            if (in_array($location, $possbile_values)) {
                update_post_meta($post_id, 'fenotices_display_location', $location);

            } else {
                update_post_meta($post_id, 'fenotices_display_location', 'beforeheader');
            }

        }

        if (!empty($_POST['fenotices_show_users'])) {
            $show_users = sanitize_text_field( wp_unslash( $_POST['fenotices_show_users'] ) );
            $possbile_values = ['allusers', 'loggedin', 'loggedout'];

            if (in_array($show_users, $possbile_values)) {
                update_post_meta($post_id, 'fenotices_show_users', $show_users);

            } else {
                update_post_meta($post_id, 'fenotices_show_users', 'allusers');
            }
        }

        if (!empty($_POST['fenotices_user_roles'])) {
            $user_roles = array_map('wp_unslash', $_POST['fenotices_user_roles']);
            $user_roles = array_map('sanitize_key', $user_roles);
            update_post_meta($post_id, 'fenotices_user_roles', $user_roles);
        }

        if (!empty($_POST['fenotices_post_types'])) {
            $fenotices_post_types = array_map('wp_unslash', $_POST['fenotices_post_types']);
            $fenotices_post_types = array_map('sanitize_key', $fenotices_post_types);
            $fenotices_post_types = array_map('sanitize_text_field', $fenotices_post_types);
            update_post_meta($post_id, 'fenotices_post_types', $fenotices_post_types);
        }

        if (!empty($_POST['fenotices_mobile_only'])) {
            $mobile_only = (bool) sanitize_text_field( wp_unslash( $_POST['fenotices_mobile_only'] ) );
            update_post_meta($post_id, 'fenotices_mobile_only', $mobile_only);
        }
    }
    
}
new fenoticesAdmin();