<?php

if (!defined('ABSPATH')) exit;

class fenoticesFrontend {
    function __construct() {
        add_action( 'template_redirect', [$this, 'get_notices'] );
    }

    function get_notices() {
        // Create a new WP_Query instance
        $args = array(
            'post_type'      => 'fen-frontendnotice',
            'posts_per_page' => -1, // Get all posts
            'fields'         => 'ids', // Only retrieve post IDs
            'post_status'    => 'publish'
        );

        $query = new WP_Query($args);

        if ($query->have_posts()) {
            // Loop through the posts
            foreach ($query->posts as $post_id) {
                $this->hook_notice($post_id);
            }
        }

        wp_reset_postdata();
    }

    function get_settings($post_id) {
        $location = get_post_meta($post_id, 'fenotices_display_location', true);
        $is_dismissable = get_post_meta($post_id, 'fenotices_notice_dismissable', true);
        $show_users = get_post_meta($post_id, 'fenotices_show_users', true);
        $user_roles = get_post_meta($post_id, 'fenotices_user_roles', true);
        $fenotices_post_types = get_post_meta($post_id, 'fenotices_post_types', true);
        $mobile_only = get_post_meta($post_id, 'fenotices_mobile_only', true);

        $arr = [];
        $arr['location'] = $location;
        $arr['is_dismissable'] = $is_dismissable;
        $arr['show_users'] = $show_users;
        $arr['user_roles'] = $user_roles;
        $arr['post_types'] = $fenotices_post_types;
        $arr['mobile_only'] = $mobile_only;

        return $arr;
    }

    function hook_notice($notice_id) {
        $settings = $this->get_settings($notice_id);

        if ($settings['show_users'] == 'loggedout' && is_user_logged_in()) {
            return;
        }

        if ($settings['show_users'] == 'loggedin' && !is_user_logged_in()) {
            return;
        }

        if (is_user_logged_in()) {
            $current_user = wp_get_current_user();
            if (!in_array('allroles', $settings['user_roles']) && empty(array_intersect($settings['user_roles'], $current_user->roles))) {
                return;
            }
        }

        if (!in_array('allroles', $settings['user_roles']) && !is_user_logged_in()) {
            return;
        }
        
        $post_type = get_post_type();

        if ($post_type == 'post' && in_array('post', $settings['post_types']) && is_home()) {
            return;
        }
        
        if (!empty($post_type) && !in_array('all', $settings['post_types']) && !in_array($post_type, $settings['post_types'])) {
            if (is_home()) {
                return;
            } 
            return;
        }

        if ($settings['mobile_only'] == true && !wp_is_mobile() ) {
            return;
        }

        $post = get_post($notice_id);
        if ($post) {
            $notice_content = apply_filters('the_content', $post->post_content);

            if ($settings['location'] == 'beforeheader') {
                $hook = 'wp_body_open';
            }

            if ($settings['location'] == 'beforewoocommercecart') {
                $hook = 'woocommerce_before_cart_table';
            }

            if ($settings['location'] == 'beforewoocommercecheckout') {
                $hook = 'woocommerce_before_checkout_form';
            }

            add_action($hook, function () use ($notice_content) {
                echo '<div class="fen-notice">'.wp_kses_post($notice_content).'</div>';
            } );

        }
    }
}

new fenoticesFrontend();